#include <vector>
#include <string>

using std::string;

/*
 * Classes to enumerate ranges of passwords.
 */
class pwd_enum_iface {
public:
	virtual void reset(int size) = 0;// Reset, giving new key size.
	virtual string next() = 0;	 // Next string, or "" if done.
};

/*
 * Enumerate a characters in a string representing a set.  Passwords are a
 * given fixed size
 */
class single_pwd_enum_list: public pwd_enum_iface {
public:
	single_pwd_enum_list(string list, int size):
		m_list(list), m_counts(size,0), m_done(false) { }
	string next();
	void reset(int size) {
		m_done = false;
		m_counts.resize(size, 0);
	}
private:
	bool m_done;			// Run is done.
	string m_list;			// Character set.  Empty for done.
	std::vector<int> m_counts;	// Position in the count.
};

/*
 * Enumerate given-size passwords from characters in a given as a range of
 * ascii values.  
 */
class single_pwd_enum_range: public pwd_enum_iface {
public:
	single_pwd_enum_range(char low, char high, int size):
		m_low(low), m_high(high), m_state(size,low), m_done(false) { }
	string next();
	void reset(int size) {
		m_done = false;
		m_state.resize(size, m_low);
	}
private:
	bool m_done;		// We are done.
	char m_low, m_high;	// Range.
	string m_state;		// Current state of the count.
};

/*
 * Enumerate through a range of sizes.  Two constructors for string as set
 * and for range of char.
 */
class pwd_enum {
public:
	// Construct an enumerator which generates all passwords made out
	// of characters appearing in the string list, of length startsize
	// through endsize.
	pwd_enum(string list, int startsize, int endsize):
		m_base_enumerator(new single_pwd_enum_list(list, startsize)),
		m_startsize(startsize), m_endsize(endsize) { }

	// Construct an enumerator which generates all passwords made out
	// of characters with code values between low and high, inclusive,
	// of length startsize through endsize.
	pwd_enum(char low, char high, int startsize, int endsize):
		m_base_enumerator
		    (new single_pwd_enum_range(low, high, startsize)),
		m_startsize(startsize), m_endsize(endsize) { }
	~pwd_enum() { delete m_base_enumerator; }

	// Get the next guess, or empty string if there are no more.
	string next() {
		// Get the next from the base enumerator.  If that ends, but
		// there are more sizes to consider, reset the enumerator for
		// the next larger size.
		string ret = m_base_enumerator->next();
		if(ret == "" && m_startsize < m_endsize) {
			m_base_enumerator->reset(++m_startsize);
			ret = m_base_enumerator->next();
		}
		return ret;
	}
private:
	int m_startsize, m_endsize;		// Password length range.
	pwd_enum_iface *m_base_enumerator;	// Does all the work.
};
